/********************************************************
-- File    : extbusdp_fpgen.c
-- Author  : Andres Farfan
-- Date    : 18/05/2000
-- Time    : 22:57
-- Descrip : extbus  module  Fig. 9.7 Pag. 644
--*********************************************************/

#include  <fpgen.h>

static void extbusregs();
static void extbustris();


static void extbusregs()
{

  DP_DEFLOFIG("extbusregs", 16, LSB_INDEX_ZERO);
    

  /* Data terminals declarations. */
  DP_LOCON( "data_e_result[15:0]", IN   , "data_e_result[15:0]" );
  DP_LOCON( "datain_sys_data[15:0]", IN , "data_sys_data[15:0]");
  DP_LOCON( "data_regwar[6:0]", OUT, "regwar[6:0]");
  DP_LOCON( "ctrl_inc_address", IN, "ctrl_inc_address");

  DP_LOCON( "ctrl_exten[4:0]", IN, "exten[4:0]");
/*  exten[4] = rdata_en
    exten[3] = addr_ld
    exten[2] = wdata_en
    exten[1] = shreg_en
    exten[0] = regwar_en*/    


  /* -- Clock Connectors */
  DP_LOCON( "extdff0_ck", IN, "dff0_ck");
  DP_LOCON( "sbrdr_ck", IN, "sbrdr_ck");
  DP_LOCON( "sbrarwar_ck", IN, "sbrarwar_ck");
  DP_LOCON( "sbwdr_ck", IN, "sbwdr_ck");
  DP_LOCON( "shfreg_ck", IN, "shfreg_ck");
  DP_LOCON( "regwar_ck", IN, "regwar_ck");  
  
  /* -- Write enable
  ** -- Enable writing when set to one*/
  DP_LOCON( "extdff0_wen", IN, "dff0_wen");
  
  
  DP_LOCON( "out_sbrdr[15:0]", OUT, "sbrdr[15:0]");
  DP_LOCON( "out_sbrarwar[15:0]", OUT, "sbrarwar[15:0]");
  DP_LOCON( "out_sbwdr[15:0]", OUT, "sbwdr[15:0]");
  DP_LOCON( "out_shfreg[3:0]", OUT, "shfreg[3:0]");
  DP_LOCON( "out_war[6:0]", OUT,  "war[6:0]");


      
  /* Power supplies terminals. */
  DP_LOCON( "vdd", IN   , "vdd" );
  DP_LOCON( "vss", IN   , "vss" );

  DP_TRACE (TRACE_ON);
      


  DP_PDFF( "dff0", 16, 0, "dff0_wen", "dff0_ck", 
                  "data_e_result[15:0]",
		  "dff0_q[15:0]",
		  "dff0_nq[15:0]", 
		  EOL);
  

/*SBRAR/WAR*/
  DP_PDFF( "sbrarwar", 16, 0, "dff2_wen", "sbrarwar_ck", 
                  "mux0_out[15:0]",
		  "sbrarwar[15:0]",
		  "n_sbrarwar[15:0]", 
		  EOL);


  DP_MUX2CS( "mux0", 16, 0,
                  "exten[3]",
		  "adder0_out[15:0]",
		  "dff0_q[15:0]",
		  "mux0_out[15:0]",
		  EOL);

  DP_ADD2F( "adder0", "sbrarwar[15:0]", 
                      "mux1_out[15:0]",
		      "nc_Cout",
		      "nc_Ovr",
		      "adder0_out[15:0]",
		      EOL);

  DP_CONST( "const2", 16, 0, "0x2", "const_two[15:0]", EOL);

  DP_MUX2CS( "mux1",  16, 0, "ctrl_inc_address", 
                  "const_two[15:0]",
		  "const_one[15:0]",
		  "mux1_out[15:0]",
		  EOL);
		  
  DP_CONST( "const1", 16, 0, "0x1", "const_one[15:0]", EOL);


  /*SBWDR*/
  DP_PDFF( "sbwdr", 16, 0, "exten[2]", "sbwdr_ck", 
                  "dff0_q[15:0]",
		  "sbwdr[15:0]",
		  "n_sbwdr[15:0]", 
		  EOL);
  

  /*SBRDR*/
  DP_PDFF( "sbrdr", 16, 0, "exten[4]", "sbrdr_ck", 
                  "data_sys_data[15:0]",
		  "sbrdr[15:0]",
		  "n_sbrdr[15:0]", 
		  EOL);
  


  /*SHFREG*/
  DP_PDFF( "shfreg", 4, 0, "exten[1]", "shfreg_ck", 
                  "dff0_q[3:0]",
		  "shfreg[3:0]",
		  "n_shfreg[3:0]", 
		  EOL);


  
  DP_BUFF( "buffshfreg", 4, 0, 1, 
                  "shfreg[3:0]", 
		  "n_shfreg[3:0]",
		  EOL);


  /*REGWAR*/
  DP_PDFF( "regwar", 7, 0, "exten[0]", "regwar_ck", 
                  "dff0_q[6:0]",
		  "regwar[6:0]",
		  "war[6:0]", 
		  EOL);
 
  
  
  DP_TRACE (TRACE_OFF);
  DP_SAVLOFIG();
}




static void  extbustris()
{

  DP_DEFLOFIG("extbustris", 16, LSB_INDEX_ZERO);
    

  /* Data terminals declarations. */
  
  DP_LOCON( "sbrdr[15:0]", IN, "sbrdr[15:0]");
  DP_LOCON( "sbrarwar[15:0]", IN, "sbrarwar[15:0]");
  DP_LOCON( "sbwdr[15:0]", IN, "sbwdr[15:0]");
  DP_LOCON( "shfreg[3:0]", IN, "shfreg[3:0]");
  DP_LOCON( "war[6:0]", IN, "war[6:0]"); 

	  
  DP_LOCON( "ctrl_extoe[6:0]", IN, "extoe[6:0]");
/*  extoe[6] = rdata_en
    extoe[5] = addr_oes
    extoe[4] = addr_oee
    extoe[3] = wdata_oes
    extoe[2] = wdata_oee
    extoe[1] = shfreg_oee
    extoe[0] = regwar_oee*/

  
  DP_LOCON( "data_sys_addr[15:0]", TRISTATE, "data_sys_addr[15:0]");
  DP_LOCON( "data_ext_rdata[15:0]", TRISTATE, "data_ext_rdata[15:0]");
  DP_LOCON( "dataout_sys_data[15:0]", OUT, "data_sys_data[15:0]");
  
  /* Power supplies terminals. */
  DP_LOCON( "vdd", IN   , "vdd" );
  DP_LOCON( "vss", IN   , "vss" );

  DP_TRACE (TRACE_ON);
      

  DP_BUSE( "ts1", 16, 0, 
                  "extoe[5]",
                  "sbrarwar[15:0]",
		  "data_sys_addr[15:0]",
		  EOL);

  DP_CONST( "constvdds", 16, 0, "0xFFFF", "const_one[15:0]", EOL);

  DP_NBUSE( "ts2", 16, 0, 
                  "extoe[4]",
                  "sbrarwar[15:0]",
		  "data_ext_rdata[15:0]",
		  EOL);


  DP_NBUSE( "ts4", 16, 0, 
                  "extoe[2]",
                  "sbwdr[15:0]",
		  "data_ext_rdata[15:0]",
		  EOL);

  DP_BUSE( "ts3", 16, 0, 
                  "extoe[3]",
                  "sbwdr[15:0]",
		  "data_sys_data[15:0]",
		  EOL);


      
  DP_NBUSE( "ts0", 16, 0, 
                  "extoe[6]",
                  "sbrdr[15:0]",
		  "data_ext_rdata[15:0]",
		  EOL);


/*sh==shfreg???*/
  DP_NBUSE( "ts6", 16, 0, 
                  "extoe[1]",
                  "constvdds[15:4],shfreg[3:0]",
		  "data_ext_rdata[15:0]",
		  EOL);

		  

  DP_NBUSE( "ts5", 16, 0, 
                  "extoe[0]",
                  "constvdds[15:7],war[6:0]",
		  "data_ext_rdata[15:0]",
		  EOL);



  DP_TRACE (TRACE_OFF);
  DP_SAVLOFIG();
}


int main()
{
  extbusregs();
  extbustris();
    

  DP_DEFLOFIG("extbusdp", 16, LSB_INDEX_ZERO);
    

  /* Data terminals declarations. */
  DP_LOCON( "data_e_result[15:0]", IN   , "data_e_result[15:0]" );
  
  DP_LOCON( "ctrl_inc_address", IN, "inc_address");
//  DP_LOCON( "ctrl_extoe[6:0]", IN, "rdata_oee,addr_oes,addr_oee,wdata_oes,wdata_oee,shfreg_oee,regwar_oee");
  DP_LOCON( "ctrl_extoe[6:0]", IN, "extoe[6:0]");
/*  extoe[6] = rdata_en
    extoe[5] = addr_oes
    extoe[4] = addr_oee
    extoe[3] = wdata_oes
    extoe[2] = wdata_oee
    extoe[1] = shfreg_oee
    extoe[0] = regwar_oee*/


//  DP_LOCON( "ctrl_exten[4:0]", IN, "rdata_en,addr_ld,wdata_en,shreg_en,regwar_en");
  DP_LOCON( "ctrl_exten[4:0]", IN, "exten[4:0]");
/*  exten[4] = rdata_en
    exten[3] = addr_ld
    exten[2] = wdata_en
    exten[1] = shreg_en
    exten[0] = regwar_en*/    


  /* -- Clock Connectors */
  DP_LOCON( "extdff0_ck", IN, "dff0_ck");
  DP_LOCON( "sbrdr_ck", IN, "sbrdr_ck");
  DP_LOCON( "sbrarwar_ck", IN, "sbrarwar_ck");
  DP_LOCON( "sbwdr_ck", IN, "sbwdr_ck");
  DP_LOCON( "shfreg_ck", IN, "shfreg_ck");
  DP_LOCON( "regwar_ck", IN, "regwar_ck");  
  
  /* -- Write enable
  ** -- Enable writing when set to one*/
  DP_LOCON( "extdff0_wen", IN, "dff0_wen");
  
  
  DP_LOCON( "data_sys_addr[15:0]", TRISTATE, "data_sys_addr[15:0]");
  DP_LOCON( "data_ext_rdata[15:0]", TRISTATE, "data_ext_rdata[15:0]");
  DP_LOCON( "data_sys_data[15:0]", INOUT, "data_sys_data[15:0]");
  DP_LOCON( "data_shfreg[3:0]", OUT, "data_shfreg[3:0]");
  DP_LOCON( "data_regwar[6:0]", OUT, "data_regwar[6:0]");
  
  /* Power supplies terminals. */
  DP_LOCON( "vdd", IN   , "vdd" );
  DP_LOCON( "vss", IN   , "vss" );

  DP_TRACE (TRACE_ON);

  /*Registers*/
  DP_IMPORT("extbusregs", "extbusregs0",
              "data_e_result[15:0]",
	      "data_sys_data[15:0]",
	      "data_regwar[6:0]",
	      "inc_address",
	      "exten[4:0]",
	      "dff0_ck",
	      "sbrdr_ck",
	      "sbrarwar_ck",
	      "sbwdr_ck",
	      "shfreg_ck",
	      "regwar_ck",
	      "dff0_wen",
	      "sbrdr[15:0]",
	      "sbrarwar[15:0]",
	      "sbwdr[15:0]",
	      "data_shfreg[3:0]",
	      "war[6:0]",
	      EOL);
	        
  /*Tristate buffers*/
  DP_IMPORT("extbustris", "extbustris",
              "sbdr[15:0]",
	      "sbrarwar[15:0]",
	      "sbwdr[15:0]",
	      "shfreg[3:0]",
	      "war[6:0]",
	      "extoe[6:0]",
	      "data_sys_addr[15:0]",
	      "data_ext_rdata[15:0]",
	      "data_sys_data[15:0]",
	      EOL);
  
  DP_TRACE (TRACE_OFF);
  DP_SAVLOFIG();
  exit(0);
}